#include <stdlib.h>
#include <iostream>
#include <time.h>

using namespace std;

static bool isSeeded = false;

class Matrice
{
	private :
		static const int MAX_RAND = 100;	
		int numLignes;
		int numColonnes;
	protected :
		double** m_matrice;
	public :
		Matrice(int m, int n)
		{
			numLignes = m;
			numColonnes = n;
			m_matrice = new double*[numLignes];
			for (int i=0;i<numLignes;i++)
				m_matrice[i] = new double[numColonnes];
			initialiserAleatoirement();
		}

		~Matrice()
		{
			for (int i=0;i<numLignes;i++)
				delete(m_matrice[i]);
			delete(m_matrice);
		}	

		//Accesseurs
		int getNumColonnes(){return numColonnes;}
		int getNumLignes(){return numLignes;}

		bool indiceLigneValide(int indice) { return !((indice < 0) || (indice >= numLignes)); }
		bool indiceColonneValide(int indice) { return !((indice < 0) || (indice >= numColonnes)); }

		bool setValeur(int ligne, int colonne, double valeur) 
		{
			if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
				return false;	
			m_matrice[ligne][colonne] = valeur;
			return true;
		} 

		double getValeur(int ligne, int colonne) 
		{
			if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
				return -1;	
			return m_matrice[ligne][colonne];
		}

		//Retourne une ligne
		double* getLigne(int indice)
		{
			if (!indiceLigneValide(indice))
				return NULL;
			double* retour = new double[numColonnes];
			for (int i=0;i<numColonnes;i++)
				retour[i] = m_matrice[indice][i];
			return retour;
		}
		//Retourne une colonne
		double* getColonne(int indice)
		{
			if (!indiceColonneValide(indice))
				return NULL;
			double* retour = new double[numLignes];
			for (int i=0;i<numLignes;i++)
				retour[i] = m_matrice[i][indice];
			return retour;
		}

		//Initialise la matrice avec des valeurs entre 0 et MAX_RAND.
		void initialiserAleatoirement()	
		{
			if (!isSeeded)
			{
				srand(time(NULL));
				isSeeded = true;
			}
			int valeur;
			for (int i=0;i<numLignes;i++)
				for (int j=0;j<numColonnes;j++)
				{
					valeur = rand() % MAX_RAND;
					m_matrice[i][j] = valeur;
				}
		}
		
		//Affichage du contenu de la matrice
		void afficherMatrice()
		{
			for (int i=0;i<numLignes;i++)
			{
				for (int j=0;j<numColonnes;j++)
				{
					cout << m_matrice[i][j] << " ";
				}
				cout << "\n";
			}
		}





};
