﻿using System;
using System.Threading;

class ManegeMon
{
    private const int nbPersonnes = 20;
    private static Manege man;

    static void Main(string[] args)
    {
        man = new Manege();

        new Thread(Voiture).Start();
        for (int i = 1; i <= nbPersonnes; i++)
            new Thread(Passager).Start(i);
    }

    private static void Passager(object no)
    {
        while (true)
        {
            man.PrendrePlace((int)no);
            man.Sortir((int)no);
        }
    }

    private static void Voiture()
    {
        while (true)
            man.FaireTours();
    }
}

class Manege
{
    private const int nbPlaces = 6, nbPersonnes = 20;
    private int personnesAssises = 0;
    private Semaphore pasPlein, peutSortir, peutPartir, peutFiler;
    private Object lockPlace;
    private Object lockSortir;
    private Object lockTours;

    public Manege()
    {
        lockPlace = new Object();
        lockSortir = new Object();
        lockTours = new Object();

        pasPlein = new Semaphore(0, nbPersonnes);
        peutSortir = new Semaphore(0, nbPlaces);
        peutPartir = new Semaphore(0, 99);
        peutFiler = new Semaphore(0, nbPlaces);
    }

    public void PrendrePlace(int no)
    {
        Monitor.Enter(lockPlace);

        if (personnesAssises >= nbPlaces)
            pasPlein.WaitOne();

        personnesAssises++;
        Console.WriteLine("Passager {0} embarque dans la voiture", no);

        if (personnesAssises == nbPlaces)
            peutPartir.Release();

        Monitor.Exit(lockPlace);
    }

    public void Sortir(int no)
    {
        Monitor.Enter(lockSortir);

        peutSortir.WaitOne();

        personnesAssises--;
        Console.WriteLine("Passager {0} sort du manège", no);

        peutFiler.WaitOne();

        Monitor.Exit(lockSortir);
    }

    public void FaireTours()
    {
        Monitor.Enter(lockTours);

        if (personnesAssises < nbPlaces)
            peutPartir.WaitOne();

        Console.WriteLine("Tous les passagers sont embarqués, la voiture fait ses tours");
        Thread.Sleep(1500);
        Console.WriteLine("Les tours sont terminés");

        peutSortir.Release(nbPlaces);
        peutFiler.Release(nbPlaces);
        pasPlein.Release(nbPlaces);

        Monitor.Exit(lockTours);
    }
}
