using System;
using System.Collections.Generic;
using System.Text;

class Matrix
{
	private const int MAX_RAND = 100;	
	private int numLignes;
    private int numColonnes;
    private double seuil;

	protected double[][] m_matrice;

	public Matrix(int m, int n)
	{
		numLignes = m;
		numColonnes = n;
		m_matrice = new double[numLignes][];
		for (int i = 0;i < numLignes; i++)
			m_matrice[i] = new double[numColonnes];
    }

    public void initialiseMatrice()
    {
        Random r = new Random();

        int valeur;
        for (int i = 0; i < numLignes; i++)
            for (int j = 0; j < numColonnes; j++)
            {
                valeur = r.Next() % MAX_RAND;
                setValeur(i, j, valeur);
            }
    }

    public void initialise(Matrix mat)
    {
        for (int i = 0; i < numLignes; i++)
            for (int j = 0; j < numColonnes; j++)
                setValeur(i, j, mat.getValeur(i, j));
    }

    public void initialiseJacobi(double borderValue, double startValue, double endValue)
    {
        int i, j;
        this.seuil = borderValue;
        for (i = 0; i < numLignes; i++)
        {
            for (j = 0; j < numColonnes; j++)
            {
                if ((i == 0 || i == numLignes - 1) || (j == 0 || j == numColonnes - 1))
                    setValeur(i, j, borderValue);
                else
                    setValeur(i, j, startValue);
            }
            if (startValue < endValue && (i != 0 && i != numLignes))
                startValue++;
        }
    }

    public int getNumColonnes() { return numColonnes; }
    public int getNumLignes()   { return numLignes; }

    public bool indiceLigneValide(int indice)   { return !((indice < 0) || (indice >= numLignes)); }
    public bool indiceColonneValide(int indice) { return !((indice < 0) || (indice >= numColonnes)); }

    public double getAbs(double valeur) { return (valeur >= 0) ? valeur : -valeur; }

    public bool setValeur(int ligne, int colonne, double valeur) 
	{
		if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
			return false;	
		m_matrice[ligne][colonne] = valeur;
		return true;
	}

    public double getValeur(int ligne, int colonne) 
	{
		if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
			return -1;	
		return m_matrice[ligne][colonne];
	}

    public double[] getLigne(int indice)
	{
		if (!indiceLigneValide(indice))
			return null;

		double[] retour = new double[numColonnes];
		for (int i=0;i<numColonnes;i++)
			retour[i] = m_matrice[indice][i];
		return retour;
	}

    public double[] getColonne(int indice)
	{
		if (!indiceColonneValide(indice))
            return null;

		double[] retour = new double[numLignes];
		for (int i=0;i<numLignes;i++)
			retour[i] = m_matrice[i][indice];
		return retour;
	}

    public double[] getVoisins(int ligne, int colonne)
    {
        double[] valeurs = new double[4];
        if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
            return null;

        valeurs[0] = (ligne == 0 || ligne == getNumLignes()-1) ? seuil : getValeur(ligne - 1, colonne);
        valeurs[1] = (ligne == 0 || ligne == getNumLignes()-1) ? seuil : getValeur(ligne + 1, colonne);
        valeurs[2] = (colonne == 0 || colonne == getNumColonnes()-1) ? seuil : getValeur(ligne, colonne - 1);
        valeurs[3] = (colonne == 0 || colonne == getNumColonnes()-1) ? seuil : getValeur(ligne, colonne + 1);
        return valeurs;
    }

    public bool epsilonAtteint(Matrix matricePrecedente, double epsilon)
    {
        double sommeDesDifferences = 0;
        for (int i = 0; i < getNumLignes(); i++)
            for (int j = 0; j < getNumColonnes(); j++)
                sommeDesDifferences += getAbs(matricePrecedente.getValeur(i, j) - getValeur(i, j));

        return sommeDesDifferences < epsilon;
    }
}
