using System;
using System.Diagnostics;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;

namespace JacobiSlave
{
    class Slave
    {
        private static JacobiRemoteService.Transfer transfer = null;
        private static ObjRef service = null;
        private static TcpChannel tcpChannel = null;
        private static int tcpPort = 1500;
        private static bool wait = false;

        public static void Main(string[] Args)
        {
            StartListen();
        }

        public static void StartListen()
        {
            StopListen();

            try
            {
                tcpChannel = new TcpChannel(tcpPort);
                ChannelServices.RegisterChannel(tcpChannel, false);

                transfer = new JacobiRemoteService.Transfer();
                service = RemotingServices.Marshal(transfer, "TestService");

                transfer.slaveCall += new JacobiRemoteService.Transfer.delegateSlaveCall(OnMasterEvent);

                Console.WriteLine("En attente d'une demande du matre, Enter pour quitter");
                Console.ReadLine();
            }
            catch (Exception Ex)
            {
                Console.WriteLine("Erreur de l'esclave:\n" + Ex.Message, "Erreur Slave");
                StopListen();
                Console.ReadLine();
            }			
        }

        public static void StopListen()
        {
            if (service != null)
                RemotingServices.Unmarshal(service);

            if (transfer != null)
                RemotingServices.Disconnect(transfer);

            if (tcpChannel != null)
                ChannelServices.UnregisterChannel(tcpChannel);

            service = null;
            transfer = null;
            tcpChannel = null;
        }

        public static JacobiRemoteService.Response OnMasterEvent(JacobiRemoteService.Request request)
        {
            JacobiRemoteService.Response response = new JacobiRemoteService.Response();
            response.slaveId = Process.GetCurrentProcess().Id;
            response.index = request.index;
            response.valeur = 0.0;

            if (request.fin)
                response.message = "Toutes les valeurs ont t calcules, limite atteinte";
            else if (wait)   // Si plusieurs Masters essaient de se connecter en mme temps
                response.message = "L'esclave est occup, ressayez plus tard";
            else
            {
                // Doit calculer ses affaires et le renvoyer
                for (int i = 0; i < 4; i++)
                    response.valeur += request.voisins[i];
                
                response.valeur /= 4;
                response.message = string.Format("Valeurs calcules: {0} + {1} + {2} + {3} / 4 = {4}", 
                    Math.Round(request.voisins[0], 4), 
                    Math.Round(request.voisins[1], 4),
                    Math.Round(request.voisins[2], 4),
                    Math.Round(request.voisins[3], 4),
                    response.valeur);

                wait = false;
            }

            Console.WriteLine(response.message);

            return response;
        }
    }
}
