/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jacobi;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Random;

/**
 *
 * @author Doum
 */
public class MatriceJacobi {
    private static final int MAX_RAND = 100;	
    private int numLignes;
    private int numColonnes;
    private int m_seuil;
    private Random randomNumber;
    private double[][] m_matrice;
    DecimalFormat format = ((DecimalFormat)NumberFormat.getInstance());
    
    public MatriceJacobi(int m, int n, int seuil)
    {
        format.setMaximumFractionDigits(4);
        format.setMinimumFractionDigits(4);
        numLignes = m;
        numColonnes = n;
        m_seuil = seuil;
        m_matrice = new double[numLignes][numColonnes];
        initialiserAleatoirement();
    }	

    //Accesseurs
    public int getNumColonnes(){return numColonnes;}
    public int getNumLignes(){return numLignes;}

    public boolean indiceLigneValide(int indice) { return !((indice < 0) || (indice >= numLignes)); }
    public boolean indiceColonneValide(int indice) { return !((indice < 0) || (indice >= numColonnes)); }

    public boolean setValeur(int ligne, int colonne, double valeur) 
    {
        if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
            return false;	
        m_matrice[ligne][colonne] = valeur;
        return true;
    } 

    public double getValeur(int ligne, int colonne) 
    {
        if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
                return -1;	
        return m_matrice[ligne][colonne];
    }

    //Initialise la matrice avec des valeurs entre 0 et MAX_RAND.
    public void initialiserAleatoirement()	
    {
        randomNumber = new Random(System.currentTimeMillis());
        int valeur;
        for (int i=0;i<numLignes;i++)
            for (int j=0;j<numColonnes;j++)
            {
                valeur = randomNumber.nextInt(MAX_RAND);
                m_matrice[i][j] = valeur;
            }
    }

    //Affichage du contenu de la matrice
    public void afficherMatrice()
    {
        for (int i=0;i<numLignes;i++)
        {
            for (int j=0;j<numColonnes;j++)
            {  
                System.out.print(format.format(m_matrice[i][j]) + " ");
            }
            System.out.println();
        }
    }
    
    public CalculJacobi getCalculJacobi(int ligne, int colonne)
    {  
        if (!indiceLigneValide(ligne) || !indiceColonneValide(colonne))
        {
            System.out.println("Indice invalide.");
            return null;
        }

        double valeur1 = (ligne == 0) ? m_seuil : getValeur(ligne-1,colonne);
        double valeur2 = (ligne + 1 == getNumLignes()) ? m_seuil : getValeur(ligne+1,colonne);	
        double valeur3 = (colonne == 0) ? m_seuil : getValeur(ligne,colonne-1);
        double valeur4 = (colonne + 1 == getNumColonnes()) ? m_seuil : getValeur(ligne,colonne+1);
        
        return new CalculJacobi(valeur1, valeur2, valeur3, valeur4, getValeur(ligne,colonne), ligne, colonne, ligne*getNumColonnes() + colonne);
    }

    public boolean epsilonAtteint(MatriceJacobi matricePrecedente,double epsilon)
    {
        double sommeDesDifferences = 0;
        for (int i=0;i<getNumLignes();i++)
            for (int j=0;j<getNumColonnes();j++)
            {
                sommeDesDifferences += getAbs(matricePrecedente.getValeur(i,j) - m_matrice[i][j]);
            }
        return sommeDesDifferences < epsilon;
    }

    double getAbs(double valeur)
    {
        return (valeur >= 0) ? valeur : -valeur;
    }
}
